#include <iostream.h>

#include <qpushbt.h>
#include <qlined.h>
#include <qcombo.h>
#include <qtooltip.h>

#include <kapp.h>
#include <kiconloader.h>
#include <klocale.h>

#include "dataclass.h"
#include "mainwidget.h"
#include "mywidget.h"
#include "dataclass.h"

#include "mocMainwidget.cpp"


extern KLocale* loc;
extern QStrList terminals;


// *********************************************************************


QMComboBox::QMComboBox (QWidget* parent, const char* name)
  : QComboBox (parent, name)
{
}


void QMComboBox::mousePressEvent (QMouseEvent* e)
{
  if (e->button () == RightButton) emit (sigRightClick());
  else QComboBox::mousePressEvent (e);
}

// *********************************************************************

QMLineEdit::QMLineEdit (QWidget* parent, const char* name)
  : QLineEdit (parent, name)
{
}

void QMLineEdit::focusOutEvent (QFocusEvent* e)
{
  QLineEdit::focusOutEvent (e);
  emit (gotFocusOutEvent ());
}

// *********************************************************************



mainWidget::mainWidget (QWidget* parent, const char* name)
  : QWidget (parent, name)
{
  // **************************  some labels  *******************************
  
  schrift = new QMLabel ("KTelnet", this);
  schrift->setFont (QFont ("Times" ,18, QFont::Bold));
  schrift->resize (120, 25);
  
  version = new QMLabel (VERSION, this);
  sessionName = new QMLabel (loc->translate("Session:"), this);
  accountname = new QMLabel (loc->translate("Account:"), this);
  hostname = new QMLabel (loc->translate("Host:"), this);

  //CT 8jan98 new items to eliminate newsession widget
  optionsLabel = new QMLabel (loc->translate("Further Options:"), this);
  
  // ***********************  the ComboBox  *********************************

  //CT 8jan98 read-write combo
  profileName = new QMComboBox (this, "profileName"); 
  //CT to see later->  profileName->setPolicy(AtTop);

  sessionManager* iter;
  for ( iter=sess.first(); iter!=0; iter=sess.next() ) 
    profileName->insertItem (iter->profileName);
  
  connect (profileName, SIGNAL (activated (int)), SLOT (changeLines ()));
  connect (profileName, SIGNAL (sigRightClick ()), 
				SLOT (changeProfileName()));
  
  QToolTip::add (profileName, 
	  loc->translate("Select session, right\nmousebutton to change name"));

  // ***********  if calling a new session a lineedit instead  **************

  newSessionLined = new QMLineEdit (this);
  QToolTip::add (newSessionLined, loc->translate("Enter new session name"));
  newSessionLined->setGeometry (90, 15, 150, 25);
  newSessionLined->hide ();
  
  // ***********************  the LineEdits  ********************************
  
  host = new QLineEdit (this, "host");
  host->setMaxLength (49);
  connect (host, SIGNAL (textChanged (const char*)), parent, 
	   SLOT (hostHasChanged (const char*)) );
  QToolTip::add (host, loc->translate("Enter remote host"));
  
  account = new QLineEdit (this, "account");
  account->setMaxLength (29);
  connect (account, SIGNAL (textChanged (const char*)), 
	   parent, SLOT (accountHasChanged (const char*)) );
  QToolTip::add (account, loc->translate("Enter account to log in"));

  // ********************** some more buttons *******************************

  lb1 = new QMLabel (loc->translate("Terminal type:"), this);
  lb2 = new QMLabel (loc->translate("Connect/Port:"), this);
  lb3 = new QMLabel (loc->translate("Textsize:"), this);
  lb4 = new QMLabel (loc->translate("Background Color:"), this);
  lb5 = new QMLabel (loc->translate("Foreground Color:"), this);

  //CT 8jan98 new button labels to avoid newsession widget
  lb6 = new QMLabel (loc->translate("Toolbar Icon:"), this);

  // the terminals are read in the constructor of mywidget
  terminal = new QComboBox (this);
  terminal->insertStrList (&terminals);

  QToolTip::add (terminal, loc->translate("Change Terminal"));

  // the textsize widget
  textsize = new QComboBox (this);
  textsize->insertItem (loc->translate("Tiny - 6x13"));
  textsize->insertItem (loc->translate("Small - 7x14"));
  textsize->insertItem (loc->translate("Medium - 8x13"));
  textsize->insertItem (loc->translate("Large - 9x15"));
  textsize->insertItem (loc->translate("Huge - 10x20"));
  QToolTip::add (textsize, loc->translate("Change terminal textsize"));

  // the connection program
  connector = new QComboBox (this);
  connector->insertItem ("Telnet");
  connector->insertItem ("rLogin");
  connector->insertItem ("Secure Shell");
  QToolTip::add (connector, loc->translate("Change connection program"));

  // the portnumber for remote machine
  port = new KIntegerLine (this);
  QToolTip::add (port, loc->translate ("Set port number, default=23"));

  bgcolorbutton = new KColorButton (this, "Change Background Color");
  QToolTip::add (bgcolorbutton, 
		 loc->translate("Click to change background color"));
  fgcolorbutton = new KColorButton (this, "Change Foreground Color");
  QToolTip::add (fgcolorbutton, 
		 loc->translate("Click to change foreground color"));

  //CT 8jan98 new lineEdits to avoid newsession widget
  options = new QLineEdit (this, "options");
  connect (options, SIGNAL (textChanged (const char*)), parent,
	   SLOT (cmdlineHasChanged (const char *)) );
  QToolTip::add (options, 
		 loc->translate("Enter additional terminal options here"));

  //CT 8jan98 new buttons to avoid newsession widget
  toolbariconbutton = new KIconLoaderButton (this);
  QToolTip::add (toolbariconbutton, "Click to change icon");
  connect (toolbariconbutton, SIGNAL (iconChanged(const char*)), 
	   SLOT (changeIcon(const char*)));

  addtotoolbar = new QPushButton(loc->translate("Add to toolbar"), 
				 this, "Add to toolbar");
  QToolTip::add (addtotoolbar, 
		 loc->translate("Click to add/remove\nsession to toolbar"));
  connect (addtotoolbar, SIGNAL (clicked()), SLOT (slotToolbar ()));

  savenewbutton = new QPushButton (loc->translate("Accept"), this, "accept");
  QToolTip::add (savenewbutton, 
		 loc->translate("Accept the new session"));
  connect (savenewbutton, SIGNAL (clicked ()), parent, 
	   SLOT(appendNewSession()) );
  savenewbutton->hide ();

  restoreoldbutton = new QPushButton(loc->translate("Cancel"), this, 
				     "Cancel");
  QToolTip::add (restoreoldbutton, 
		 loc->translate("Cancel entering a new session"));
  connect (restoreoldbutton, SIGNAL(clicked()), parent, 
	   SLOT(restoreSession ()));
  restoreoldbutton->hide ();

  // **********************  the connect button  *****************************
  
  Connect = new QPushButton (loc->translate("Connect"), this, "Connect");
  connect (Connect, SIGNAL (clicked ()), parent, SLOT (startSession ()) ); 
  Connect->setDefault (TRUE);
  QToolTip::add (Connect, loc->translate("Start the remote session"));

  // ************************  the help button  ******************************

  helpbutton = new QPushButton (loc->translate("Help"), this, "Help");
  QToolTip::add (helpbutton, loc->translate("General help on Remote Login"));
  connect (helpbutton, SIGNAL (clicked ()), SLOT (showHelper ()) );

  // ******************** resize the widget over buttons ********************
  
  buttonUp = new QPushButton (this);
  buttonUp->setPixmap (KApplication::getKApplication()->getIconLoader()->
		       loadIcon("smallup.xpm"));

  buttonDown = new QPushButton (this);
  buttonDown->setPixmap (KApplication::getKApplication()->getIconLoader()->
			 loadIcon("smalldown.xpm"));

  QToolTip::add (buttonUp, loc->translate("Lower Window Size"));
  QToolTip::add (buttonDown, loc->translate("Raise Window Size"));

  // ***************************** some focusPolicy *************************

  connect (this, SIGNAL (showOptionsPopup(QPoint)), parent, 
	   SLOT (showOptionsPopup(QPoint)));
  connect (this, SIGNAL (showFilePopup(QPoint)), parent, 
	   SLOT (showFilePopup(QPoint)));

  connect (buttonUp, SIGNAL (clicked ()), SLOT (reduceSize ()));
  connect (buttonDown, SIGNAL (clicked ()), SLOT (increaseSize ()));

  connect (terminal, SIGNAL (activated (int)), parent, 
	   SLOT (terminalHasChanged(int)));
  connect (textsize, SIGNAL (activated (int)), parent, 
	   SLOT (textsizeHasChanged(int)));
  connect (connector, SIGNAL (activated (int)), parent, 
	   SLOT (connectorHasChanged(int)));
  connect (connector, SIGNAL (activated (int)), 
	   SLOT (checkConnectorForPort (int)));
  connect (fgcolorbutton, SIGNAL (changed (const QColor&)), parent, 
	   SLOT (fgcolorHasChanged (const QColor&)));
  connect (bgcolorbutton, SIGNAL (changed (const QColor&)), parent, 
	   SLOT (bgcolorHasChanged (const QColor&)));
  connect (port, SIGNAL (textChanged (const char*)), parent, 
	   SLOT (portHasChanged (const char*)));

  connect (this, SIGNAL (updateRects ()), parent, SLOT (updateRects ()));

  connect (this, SIGNAL (addToToolbar ()), parent, SLOT (addToToolbar ()));
  connect (this, SIGNAL (removeFromToolbar ()), parent, 
	   SLOT (removeFromToolbar ()));
  
  connect (this, SIGNAL (needSessionSaving (bool)), parent, 
	   SLOT (needSessionSaving (bool)));

  connect (this, SIGNAL (deactivateSLOTS ()),parent, SLOT(deactivateSLOTS ()));
  connect (this, SIGNAL (activateSLOTS ()), parent, SLOT (activateSLOTS ()));

  setFixedSize (375, 125);

  Connect->setGeometry (270, 15, 80, 25);
  schrift->move (268, 85);
  version->move (340, 84);
  sessionName->move (20, 12);
  accountname->move (20, 82);
  hostname->move (20, 47);
  host->setGeometry (90, 50, 150, 25);
  account->setGeometry (90, 85, 150, 25);
  profileName->setGeometry (90, 15, 150, 25);    // vorher alles y -= 5
  //CT 8jan98
  optionsLabel->setGeometry (20, 294, 157, 25);
  options->setGeometry (150, 294, 200, 25);

  lb1->setGeometry (20, 119, 130, 25);
  lb3->setGeometry (20, 154, 130, 25);
  lb2->setGeometry (20, 189, 130, 25);
  lb4->setGeometry (20, 224, 150, 25);
  lb5->setGeometry (20, 259, 150, 25);
  //CT 8jan98
  lb6->setGeometry (20, 329, 150, 25);

  terminal->setGeometry (120, 120, 120, 25);
  textsize->setGeometry (120, 155, 120, 25);
  connector->setGeometry (120, 190, 120, 25);
  port->setGeometry (270, 190, 80, 25);
  fgcolorbutton->setGeometry (160, 260, 80, 25);
  bgcolorbutton->setGeometry (160, 225, 80, 25);

  buttonUp->setGeometry (360, 14, 13, 13);
  buttonDown->setGeometry (360, 27, 13, 13);

  //CT 8jan98
  toolbariconbutton->setGeometry (150, 329, 35, 35);

  //CT 8jan98
  addtotoolbar->setGeometry (210, 334, 110, 25);
  savenewbutton->setGeometry (270, 120, 80, 25);
  restoreoldbutton->setGeometry (270, 155, 80, 25 );
  helpbutton->setGeometry (270, 50, 80, 25 );

  // some keyboard accelerators
  acc = new QAccel (this);
  acc_updown = new QAccel (this);
  acc->connectItem (acc->insertItem (Key_PageUp), this, SLOT (reduceSize()));
  acc->connectItem (acc->insertItem (Key_PageDown), this,  
		    SLOT (increaseSize()));

  // these accelerators can not be called when changing the session name
  acc_updown->connectItem (acc_updown->insertItem (Key_Return), parent, 
			   SLOT (startSession()));
  acc_updown->connectItem (acc_updown->insertItem (Key_Up), this, 
			   SLOT (changeUp ()));
  acc_updown->connectItem (acc_updown->insertItem (Key_Down), this, 
			   SLOT (changeDown()));
  acc_updown->connectItem (acc_updown->insertItem (ALT+Key_S), this, 
			   SLOT (changeProfileName ()));
}


mainWidget::~mainWidget ()
{
  delete Connect;
  delete schrift;
  delete version;
  delete sessionName;
  delete accountname;
  delete hostname;
  delete host;
  delete account;
  delete profileName;
  delete port;
  delete newSessionLined;
  delete lb1;
  delete lb2;
  delete lb3;
  delete lb4;
  delete lb5;
  delete buttonUp;
  delete buttonDown;
  //CT 9jan98
  delete lb6;
  delete addtotoolbar;
  delete toolbariconbutton;
  delete savenewbutton;
  delete restoreoldbutton;
  delete helpbutton;
}


// **************** some slots for changing profile name *********************


void mainWidget::changeProfileName ()
  {
    emit (deactivateSLOTS ());

    profileName->hide ();
    QToolTip::add (newSessionLined, 
	  loc->translate("Enter new session name.\nPress RETURN to finish."));
    newSessionLined->show ();
    newSessionLined->setFocus ();
    newSessionLined->setText (sess.current()->profileName);
    newSessionLined->selectAll ();
    connect (newSessionLined, SIGNAL (returnPressed()), 
	     SLOT (profileNameChanged()));
    connect (newSessionLined, SIGNAL (textChanged(const char*)), 
	     SLOT (needSessionSaving (const char*)));
    connect (newSessionLined, SIGNAL (gotFocusOutEvent()), 
	     SLOT (profileNameChanged ()));
    
    // disconnect the keyboard accelerators
    acc_updown->setEnabled (FALSE);
  }


void mainWidget::profileNameChanged ()
{
  emit (activateSLOTS ());

  newSessionLined->hide ();
  QToolTip::add (newSessionLined, loc->translate("Enter new session name"));
  profileName->show ();

  // change everything, but only if needed
  if (strcmp (sess.current()->profileName, newSessionLined->text()) != 0)
    {
      strcpy (sess.current()->profileName, newSessionLined->text());

      profileName->changeItem (sess.current()->profileName, 
			       profileName->currentItem ());
      
      if (sess.current()->addToToolbar == TRUE)
	{
	  emit (addToToolbar ());
	}
    }
  
  // disconnect the signals/slots
  disconnect (newSessionLined, SIGNAL (returnPressed()), this, 
	      SLOT (profileNameChanged()));  
  disconnect (newSessionLined, SIGNAL (textChanged(const char*)), this, 
	      SLOT (needSessionSaving (const char*)));
  disconnect (newSessionLined, SIGNAL (gotFocusOutEvent()), this,
	      SLOT (profileNameChanged ()));

  if (needSaving == TRUE) emit (needSessionSaving (TRUE));

  // reconnect the keyboard accelerators
  acc_updown->setEnabled (TRUE);
}


// a slot connected from the newSessionLined in the upper two functions
void mainWidget::needSessionSaving (const char*)
{
  needSaving = TRUE;
}


// ****************************************************************************


void mainWidget::checkConnectorForPort (int)
{
  if (strcmp(connector->currentText(), "rLogin") == 0) 
    port->setEnabled (FALSE);
  else port->setEnabled (TRUE);
}


void mainWidget::changeIcon (const char* newIcon)
{
  strcpy (sess.current()->toolbarIcon, newIcon);
  needSessionSaving (TRUE);

  // check if also needs to be changed in toolbar
  if (sess.current()->addToToolbar == TRUE)
    {
      emit(addToToolbar ());      
    }
}


void mainWidget::changeLines ()          // another session is desired
  {
    // here of course everything changes but should not be stored
    onlyChangeLines = TRUE;
    bool _neededSaving_ = needSaving;
    
    sess.at (profileName->currentItem ());

    account->setText (sess.current()->account);
    host->setText (sess.current()->host);

    // reset the terminal type
    terminal->setCurrentItem (terminals.find(sess.current()->terminal));
    
    // reset the connection program
    if (strcmp("telnet", sess.current()->connectionProgram) == 0) 
      connector->setCurrentItem (0);
    if (strcmp("rlogin", sess.current()->connectionProgram) == 0) 
      {
	connector->setCurrentItem (1);
	port->setEnabled (FALSE);
      }
    else port->setEnabled (TRUE);
    if (strcmp("ssh", sess.current()->connectionProgram) == 0) 
      connector->setCurrentItem (2);
    
    // reset the fontsize
    if (strcmp("6x12", sess.current()->font) == 0) 
      textsize->setCurrentItem (0);
    if (strcmp("7x14", sess.current()->font) == 0) 
      textsize->setCurrentItem (1);
    if (strcmp("8x13", sess.current()->font) == 0) 
      textsize->setCurrentItem (2);
    if (strcmp("9x15", sess.current()->font) == 0) 
      textsize->setCurrentItem (3);
    if (strcmp("10x20", sess.current()->font) == 0) 
      textsize->setCurrentItem (4);

    // the port number
    port->setValue(sess.current()->port);
    
    // set the colors
    fgcolorbutton->setColor (string2QColor(sess.current()->fgcolor));
    bgcolorbutton->setColor (string2QColor(sess.current()->bgcolor));

    // the options line
    options->setText (sess.current()->commandLine);

    // set the icon
    toolbariconbutton->setIcon(sess.current()->toolbarIcon);

    if (sess.current()->addToToolbar == TRUE)
      addtotoolbar->setText (loc->translate("Remove"));
    else addtotoolbar->setText (loc->translate("Add to toolbar"));
    
    // store back and make shure the "save" buttons are in the correct state
    needSaving = _neededSaving_;
    needSessionSaving (needSaving);
    onlyChangeLines = FALSE;
  }


void mainWidget::getMouseEvent (QMouseEvent* e)
{
  mousePressEvent (e);
}


//
// if the fontsize is changed this font should not be changed at all
//

void mainWidget::paintEvent (QPaintEvent*)
  {
    if (schrift->font()== QFont ("Times" ,18, QFont::Bold));
    else schrift->setFont (QFont ("Times" ,18, QFont::Bold));
  }  


void mainWidget::mousePressEvent (QMouseEvent* e)
{
  if (e->button () == RightButton) emit ( showOptionsPopup (e->pos()) );
  if (e->button () == LeftButton) emit ( showFilePopup (e->pos ()) );
}


void mainWidget::increaseSize ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");

  if (height() == 125) 
    {
      showFull ();
      k->writeEntry ("windowSize", "full");
    }
  else if (height() == 50) 
    {
      showStandard ();
      k->writeEntry ("windowSize", "standard");
    }
}


void mainWidget::reduceSize ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");

  if (height() == 125) 
    {
      showMinimum () ;
      k->writeEntry ("windowSize", "minimum");
    }
  else if (height() == 375) 
    {
      showStandard ();
      k->writeEntry ("windowSize", "standard");
    }
}


// slots for the keyboard accelerator

void mainWidget::changeUp ()
{
  if (profileName->currentItem() > 0)
    {
      profileName->setCurrentItem (profileName->currentItem()-1);
      changeLines ();
    }
}


void mainWidget::changeDown ()
{
  if (profileName->currentItem() < profileName->count()-1) 
    {
      profileName->setCurrentItem (profileName->currentItem()+1);
      changeLines ();
    }
}


// ****************************************************************************


void mainWidget::showMinimum ()
{
  setFixedSize (375, 50);

  Connect->setGeometry (270, 15, 80, 25);
  sessionName->move (20, 12);
  profileName->setGeometry (90, 15, 150, 25);

  hostname->move (20, 60);
  host->setGeometry (90, 60, 150, 25);

  buttonUp->setPixmap (KApplication::getKApplication()->getIconLoader()->loadIcon("smallup_grey.xpm"));
  buttonDown->setPixmap (KApplication::getKApplication()->getIconLoader()->loadIcon("smalldown.xpm"));

  emit (updateRects());
}


void mainWidget::showStandard ()
{
  setFixedSize (375, 125);

  // push them back to a nice site
  Connect->setGeometry (270, 15, 80, 25);
  hostname->move (20, 47);
  host->setGeometry (90, 50, 150, 25);
  sessionName->move (20, 12);
  profileName->setGeometry (90, 15, 150, 25);    // vorher alles y -= 5

  lb1->setGeometry (20, 140, 130, 25);
  terminal->setGeometry (120, 140, 120, 25);

  buttonUp->setPixmap (KApplication::getKApplication()->getIconLoader()->loadIcon("smallup.xpm"));
  buttonDown->setPixmap (KApplication::getKApplication()->getIconLoader()->loadIcon("smalldown.xpm"));

  helpbutton->show();

  emit (updateRects());
}


void mainWidget::showFull ()
{
  setFixedSize (375, 375);

  lb1->setGeometry (20, 119, 130, 25);
  terminal->setGeometry (120, 120, 120, 25);

  hostname->move (20, 47);
  host->setGeometry (90, 50, 150, 25);

  buttonUp->setPixmap (KApplication::getKApplication()->getIconLoader()->loadIcon("smallup.xpm"));
  buttonDown->setPixmap (KApplication::getKApplication()->getIconLoader()->loadIcon("smalldown_grey.xpm"));

  helpbutton->show();

  emit (updateRects());
}




// ****************************************************************************
//
//  Inherited QLabel class implementation to receive mousePressEvents on
//  labels. The event is parsed to the parent widget.
//
// ****************************************************************************


QMLabel::QMLabel (const char* text, QWidget* parent)
  : QLabel (text, parent)
{
  parentW = (mainWidget*)parent;
}


void QMLabel::mousePressEvent (QMouseEvent* e)
{
  //
  // you must correct for the position of the Label
  //

  QMouseEvent e1 (Event_MouseButtonPress, 
		  e->pos()+pos(), e->button(), e->state());

  parentW->getMouseEvent (&e1);
}


//CT 9jan98 two slots for the new buttons
void mainWidget::showHelper ()
  {
    KApplication::getKApplication()->
      invokeHTMLHelp ("",(height()<=125)?"":"newsession");
  }


void mainWidget::slotToolbar ()
{
  if (strcmp(addtotoolbar->text(), loc->translate("Add to toolbar")) == 0)
    {
      addtotoolbar->setText(loc->translate("Remove"));
      emit(addToToolbar ());
    }
  else 
    {
      addtotoolbar->setText(loc->translate("Add to toolbar"));
      emit (removeFromToolbar ());
    }
}


void mainWidget::restoreEvent (QCloseEvent*)
  {
    //CT to be seen, in correlation with cancelNewSession
    //myparent->cancelNewSession ();
  }


sessionManager* mainWidget::getNewSession ()
  {
    sessionManager* newSess = new sessionManager;

    strcpy (newSess->commandLine, options->text ());
    strcpy (newSess->terminal, terminal->currentText ());

    if (textsize->currentItem () == 0) strcpy (newSess->font, "6x12");
    if (textsize->currentItem () == 1) strcpy (newSess->font, "8x13");
    if (textsize->currentItem () == 2) strcpy (newSess->font, "7x14");
    if (textsize->currentItem () == 3) strcpy (newSess->font, "9x15");
    if (textsize->currentItem () == 4) strcpy (newSess->font, "10x20");

    if (connector->currentItem () == 0)
      strcpy (newSess->connectionProgram, "telnet");
    if (connector->currentItem () == 1)
      strcpy (newSess->connectionProgram, "rlogin");
    if (connector->currentItem () == 2)
      strcpy (newSess->connectionProgram, "ssh");

    newSess->port = port->value();

    if (strcmp(addtotoolbar->text(), loc->translate("Add to toolbar")) == 0)
      newSess->addToToolbar = FALSE;
    else newSess->addToToolbar = TRUE;

    strcpy (newSess->toolbarIcon, toolbariconbutton->icon ());

    strcpy (newSess->profileName, newSessionLined->text ());
    strcpy (newSess->host, host->text ());
    strcpy (newSess->account, account->text ());
    strcpy (newSess->fgcolor, QColor2string(fgcolorbutton->color()));
    strcpy (newSess->bgcolor, QColor2string(bgcolorbutton->color()));

    return (newSess);
  }


