#ifndef __MAINWIDGET_H
#define __MAINWIDGET_H


#include <qdialog.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qlined.h>
#include <qaccel.h>

#include <kcolorbtn.h>
#include <kiconloaderdialog.h>
#include <kintegerline.h>

#include "mywidget.h"
#include "dataclass.h"


class QPushButton;
class QComboBox;
class QLineEdit;
class mainWidget;



//
// This class must be inherited to get a QLabel which does parse any
// received mousePressEvents to its parent widget. The parent widget 
// is mainWidget in this case.
//
// mousePressEvent must of course correct the mousePosition for the
// parent widget
//

class QMLabel : public QLabel
{
  public:
    QMLabel (const char* text=0, QWidget* parent=0);

  protected:
    void mousePressEvent (QMouseEvent* e);
    
    mainWidget* parentW;
};



// needed for accepting right mousebutton clicks to change session name

class QMComboBox : public QComboBox
{
Q_OBJECT
public:
  QMComboBox (QWidget* parent=0, const char* name=0);
protected:
  void mousePressEvent (QMouseEvent* e);
signals:
  void sigRightClick ();
};



// needed for receiving focusOutEvents

class QMLineEdit : public QLineEdit
{
Q_OBJECT
public:
  QMLineEdit (QWidget* parent=0, const char* name=0);
protected:
  void focusOutEvent (QFocusEvent* e);
signals:
  void gotFocusOutEvent ();
};



class mainWidget : public QWidget
{
  Q_OBJECT
  friend class MyWidget;
  public:
    mainWidget (QWidget* parent=0, const char* name=0);
    ~mainWidget ();
    void getMouseEvent (QMouseEvent* e);

  public slots:
    void showMinimum ();
    void showStandard ();
    void showFull ();
    void changeLines ();
    sessionManager* getNewSession (); //CT 9jan98

    void changeUp ();
    void changeDown ();

    // the first is called before, the other after changing the profileName
    void profileNameChanged ();
    void changeProfileName ();

    // emitted sometimes when session saving is needed (argument is dummy, 
    // not needed but signals wants it)
    void needSessionSaving (const char*);

  private slots:
    void increaseSize ();
    void reduceSize ();
    void changeIcon (const char*);
    void checkConnectorForPort (int);

  protected slots:
    void showHelper ();
    void slotToolbar ();

  protected:
    void paintEvent (QPaintEvent*);
    void mousePressEvent (QMouseEvent*);

    void restoreEvent (QCloseEvent*); //CT 9jan98


  signals:
    void showOptionsPopup (QPoint);
    void showFilePopup (QPoint);
    void updateRects ();
    void addToToolbar ();
    void removeFromToolbar ();
    void needSessionSaving (bool);
    void deactivateSLOTS ();
    void activateSLOTS ();

  private:
    QAccel* acc;
    QAccel* acc_updown;

    QMLabel* schrift;
    QMLabel* version;
    QMLabel* sessionName;
    QMLabel* accountname;
    QMLabel* hostname;
    //CT 8jan98
    QMLabel* optionsLabel;

    QPushButton* Connect;		// the connect - Button
    QMComboBox* profileName;
    QMLineEdit* newSessionLined;         // a lineedit to enter a new session

    QLineEdit* host; 
    QLineEdit* account;
    //CT 8jan98
    QLineEdit* options;

    QMLabel* lb1;
    QMLabel* lb2;
    QMLabel* lb3;
    QMLabel* lb4;
    QMLabel* lb5;
    //CT 8jan98
    QMLabel* lb6;

    QComboBox* terminal;
    QComboBox* connector;
    QComboBox* textsize;
    
    KIntegerLine* port;

    KColorButton* fgcolorbutton;
    KColorButton* bgcolorbutton;

    //CT 8jan98
    KIconLoaderButton* toolbariconbutton;

    QPushButton* buttonUp;
    QPushButton* buttonDown;
    //CT 8jan98
    QPushButton* addtotoolbar;
    QPushButton* savenewbutton;
    QPushButton* restoreoldbutton;
    QPushButton* helpbutton;
};



#endif









