#include <unistd.h>
#include <string.h>
#include <iostream.h>
#include <stdlib.h>

#include <qpushbt.h>
#include <qlabel.h>
#include <qcombo.h>
#include <qlined.h>
#include <qpixmap.h>
#include <qfiledlg.h>
#include <qstring.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qmsgbox.h>
#include <qtimer.h>
#include <qslider.h>
#include <qlcdnum.h>
#include <qpalette.h>
#include <qframe.h>
#include <qtooltip.h>
#include <qkeycode.h>

#include <ksimpleconfig.h>
#include <kconfigbase.h>
#include <kapp.h>
#include <kmenubar.h>
#include <kcolordlg.h>
#include <kiconloader.h>
#include <kiconloaderdialog.h>
#include <klocale.h>

#include "mywidget.h"
#include "mainwidget.h"

#include "dataclass.h"
#include "configTab.h"

#include "mocMywidget.cpp"


#define OPTS_TOOLBAR_ID 111      // an ID for a popupmenu
#define OPTS_TOOLBAR_POS 1


// 
// a list of terminals read out of the resource-file and their prefered
// commandline options that differ from terminal to terminal
//
QStrList terminals;
QStrList termFgColor;
QStrList termBgColor;
QStrList termFontSize;


// the translation to native language
extern KLocale* loc;


//
// also defined (as extern) in dataclass.h - the main variables
//
QList<sessionManager> sess;

//
// stores wheather sessions have to be saved or not. As this will be set by
// all lineedits when the session just changes as the contents will change
// then too, I need another variable to check this. It will be set by
// changeLines of mainwidget.
//
bool needSaving;
bool onlyChangeLines = FALSE;
bool neededSavingbefore;


// *******************  end of the variable section  ***********************


//
// some helper functions to transform QColors to strings and vice versa
//


// color must be of the following form: "#aabbcc" (hex-code)

QColor string2QColor (const char* color)
{
  // set the colors
  
  char help[10];
  char help1[4];

  int red=0, green=0, blue=0;
  
  strcpy (help, color);
  strcpy (help, help+1);              // delete first character (#)
  strncpy (help1, help, 2);
  help1[2] = '\0';                    // set the \0 on end   
  sscanf (help1, "%x", &red);         // hexadecimal
  if (red>255) red = 255;             // check the colors
  
  strcpy (help, help+2);              // delete red part (2 characters)
  strncpy (help1, help, 2);
  help1[2] = '\0';
  sscanf (help1, "%x", &green);
  if (green>255) green = 255;
  
  strcpy (help, help+2);              // delete green part
  strncpy (help1, help, 2); 
  help1[2] = '\0';
  sscanf (help1, "%x", &blue);        // the rest is blue
  if (blue>255) blue = 255;

  return (QColor (red, green, blue));
}


// returns a color in the following form: "#aabbcc" (hex-code)

char* QColor2string (QColor col)
{
  static char colour[10];
  int red, green, blue;
  char help[10];

  red = col.red ();
  green = col.green ();
  blue = col.blue ();
  
  strcpy (colour, "#");
  
  sprintf (help, "%x", red);
  if (red < 16) strcat (colour, "0");
  strcat (colour, help);
  
  sprintf (help, "%x", green);
  if (green < 16) strcat (colour, "0");
  strcat (colour, help);
  
  sprintf (help, "%x", blue);
  if (blue < 16) strcat (colour, "0");
  strcat (colour, help);
  
  return colour;
}


//
// find a new session ID, needed for entries into toolbar
// IDs start on 100 up
//

int getNewSessionID ()
{
  sessionManager* iter;
  int newID = 99;
  int oldCurrent = sess.at();
  bool foundNewID;

  do
    {
      foundNewID = TRUE;
      newID++;
      for (iter=sess.first(); iter!=0; iter=sess.next())
	{
	  if (iter->ID == newID) 
	    {
	      foundNewID = FALSE;
	      break;
	    }
	}
    }  
  while (foundNewID == FALSE);

  sess.at (oldCurrent);

  cout << "Found new ID " << newID << endl;

  return (newID);
}


//
// ************  implementation of MyWidget - the main widget  ************
//

//
// CT 9jan98 old cancelNewSession replaced!!!
// This is called when the new session dialog exits to restore the
// window that way it looked before "New Session" was pressed
//
void MyWidget::restoreSession ()
{
  m->newSessionLined->hide ();
  m->profileName->show ();
  m->restoreoldbutton->hide ();
  m->savenewbutton->hide ();
  m->Connect->show ();
  
  activateSLOTS ();
 
  connect (m->toolbariconbutton, SIGNAL (iconChanged(const char*)), m,
	      SLOT (changeIcon(const char*)));

  // reconnect the resize buttons - display the correct pixmaps
  connect (m->buttonUp, SIGNAL (clicked ()), m, SLOT (reduceSize ()));
  connect (m->buttonDown, SIGNAL (clicked ()), m, SLOT (increaseSize ()));
  m->buttonUp->setPixmap (KApplication::getKApplication()->getIconLoader()->loadIcon("smallup.xpm"));


  disconnect (m->addtotoolbar, SIGNAL (clicked()), this, 
	      SLOT (slotToolbar ()));
  connect (m->addtotoolbar, SIGNAL (clicked()), m, SLOT (slotToolbar ()));

  m->profileName->setCurrentItem (sess.find(prev.session));
  
  if (prev.status == 1) m->showStandard();
  else if (prev.status == 2) m->showFull();
  else m->showMinimum();

  if (onlyChangeLines == TRUE) 
    {
      onlyChangeLines = FALSE;
      needSessionSaving (neededSavingbefore);
    }

  m->changeLines();

  delete NewSessAccel;

  editNewSession = FALSE;
} 


void MyWidget::deactivateSLOTS ()
{
  filemenue->setItemEnabled (filemenue->idAt(0), FALSE);
  filemenue->setItemEnabled (filemenue->idAt(1), FALSE);
  filemenue->setItemEnabled (filemenue->idAt(2), FALSE);

  tools->setItemEnabled (1, FALSE);
  tools->setItemEnabled (2, FALSE);
  tools->setItemEnabled (3, FALSE);  

  // reconnect the keyboard accelerators
  m->acc_updown->setEnabled (FALSE);
  m->acc->setEnabled (FALSE);
}


void MyWidget::activateSLOTS ()
{
  filemenue->setItemEnabled (filemenue->idAt(0), TRUE);
  filemenue->setItemEnabled (filemenue->idAt(1), TRUE);
  if (needSaving == TRUE)
    filemenue->setItemEnabled (filemenue->idAt(2), TRUE);

  tools->setItemEnabled (1, TRUE);
  tools->setItemEnabled (2, TRUE);
  if (needSaving == TRUE) tools->setItemEnabled (3, TRUE);

  // reconnect the keyboard accelerators
  m->acc_updown->setEnabled (TRUE);
  m->acc->setEnabled (TRUE);
}


// ***************************  private member  ****************************

// the configuration file will be read in your home-dir, if not possible
// it will be opened in tellys source directory.

void MyWidget::readConfigurationFile ()
  {
    KConfig* kc = KApplication::getKApplication()->getConfig();
    kc->setGroup ("defaults");
    QString defaultTerminal = kc->readEntry ("defaultTerminal");

    KSimpleConfig* k = new KSimpleConfig ( (const char*)
			   (kapp->localconfigdir() + QString("/") + 
			    SESSIONSFILE) );

    QString s;
    char item[20];
    char s1 [4];

    do
      {
	//
	// append the list of sessions (the list starts with index 0)
	//
	sess.append( new sessionManager );

	//
	// choose the item to load
	//
	strcpy (item, "session");
	sprintf ( s1, "%i", sess.at() );
	strcat (item, s1);
	k->setGroup (item);

	//
	// now read the sessions from file
	//
	s = k->readEntry ("host","-");
	strcpy (sess.current()->host, (const char*)s);
	
	s = k->readEntry ("account","-");
	strcpy (sess.current()->account, (const char*)s);

	s = k->readEntry ("font","9x15");
	strcpy (sess.current()->font, (const char*)s);

	s = k->readEntry ("terminal",(const char*)defaultTerminal);
	strcpy (sess.current()->terminal, (const char*)s);

	s = k->readEntry ("connection","telnet");
	strcpy (sess.current()->connectionProgram, (const char*)s);

	sess.current()->port = k->readNumEntry ("port", 23);

	s = k->readEntry ("bgcolor","#ffffff");
	strcpy (sess.current()->bgcolor, (const char*)s);

	s = k->readEntry ("fgcolor","#000000");
	strcpy (sess.current()->fgcolor, (const char*)s);

	s = k->readEntry ("options", "-ls");
	strcpy (sess.current()->commandLine, (const char*)s);

	s = k->readEntry ("addToToolbar", "TRUE");
	if (s == QString("TRUE"))
	  sess.current()->addToToolbar = TRUE;
	else sess.current()->addToToolbar = FALSE;

	s = k->readEntry ("toolbarIcon", "contents.xpm");
	strcpy (sess.current()->toolbarIcon, (const char*)s);

	sess.current()->ID = k->readNumEntry ("ID", 100+sess.at());  

	s = k->readEntry ("profile");
	strcpy (sess.current()->profileName, (const char*)s);
      }
    while ( s.isEmpty() == FALSE );

    //
    // if no session is available at least the ordinary terminal will be 
    // displayed
    //
    if (sess.count() == 1)
      strcpy (sess.current()->profileName, loc->translate("ordinaryTerminal"));
    else sess.removeLast();

    delete k;
  }


// ***********************  save to resource file  ****************************


void MyWidget::saveConfig ()
  {
    //
    // now rewrite the configuration
    //
    KSimpleConfig* k = new KSimpleConfig ( (const char*)
			   (kapp->localconfigdir() + QString("/") + 
			    SESSIONSFILE) );

    char s1[5];
    char item[20];
    int i=0;

    //
    // first delete the whole file
    //
    do
      {
	//
	// choose the item to remove
	//
	strcpy (item, "session");
	sprintf ( s1, "%i", i++ );
	strcat (item, s1);
	k->setGroup (item);
      }
    while (k->deleteGroup (item) == TRUE);
    
    int oldCurrent = sess.at ();
    //
    // now write it back to disk
    //
    sessionManager* iter;
    for ( iter=sess.first(); iter!=0; iter=sess.next() )
      {
	strcpy (item, "session");
	sprintf ( s1, "%i", sess.find(iter) );
	strcat (item, s1);

	k->setGroup (item);
	
	k->writeEntry ("profile", iter->profileName);
	k->writeEntry ("host", iter->host);
	k->writeEntry ("account", iter->account);
	k->writeEntry ("font", iter->font);
	k->writeEntry ("terminal", iter->terminal);
	k->writeEntry ("connection", iter->connectionProgram);
	k->writeEntry ("port", iter->port);
	k->writeEntry ("bgcolor",  iter->bgcolor);
	k->writeEntry ("fgcolor", iter->fgcolor);
	k->writeEntry ("options", iter->commandLine);
	k->writeEntry ("toolbarIcon", iter->toolbarIcon);
	k->writeEntry ("ID", iter->ID);
	
	if (iter->addToToolbar == TRUE)
	  k->writeEntry ("addToToolbar", "TRUE");
	else k->writeEntry ("addToToolbar", "FALSE");
      }

    k->sync ();

    delete k;

    sess.at (oldCurrent);

    needSessionSaving (FALSE);
  }


void MyWidget::saveConfiguration ()
  {
    // just parses to saveConfig ()
    saveConfig ();
  }


// *************************  delete a session  ******************************

// **********  these slots contain to buttons of the NewSession widget  ******

// i know, no brilliant solution ...


void MyWidget::deleteSession ()
  {
    needSessionSaving (TRUE);

    // now the combobox must be aktualisiert ...
    m->profileName->removeItem (sess.at());
    
    // remove the certain ID from toolbar
    tools->removeItem (sess.current()->ID);

    // check for deleting the last entry
    if (m->profileName->count() <= m->profileName->currentItem()) 
      m->profileName->setCurrentItem (m->profileName->count()-1);

    // now the session list
    if (uint(sess.at()) == sess.count()-1) 
      {
	// this is the last entry
	sess.at (sess.count()-2);
	
	sess.removeLast ();
      }
    // remove the current list entry
    else sess.remove ();
    
    m->changeLines ();
  }


// ************************  append a new session  ***************************


// a small helper for finding the whitesapces in the strings (they are not 
// allowed)

unsigned int strpos (const char* a, const char b)
  {
    return ( strchr (a, b) - a + 1 );
  } 


bool MyWidget::appendNewSession ()
  {
    // Falsch-Eingaben abfangen ...

    sessionManager* newSess = m->getNewSession ();

    if ((strlen (newSess->profileName) == 0)  
	||  (strlen (newSess->host) == 0)  )
      {
	QMessageBox::warning (0, loc->translate("Warning"), 
			      loc->translate("Nothing to append!"));
   
	delete newSess;
        return (FALSE);
      }

    // are there any white spaces in the strings (not allowed) ?
    
    if ( ( strpos (newSess->host, ' ') <= strlen (newSess->host) )  ||
	 ( strpos (newSess->account, ' ') <= strlen (newSess->account) ) )
      {
	QMessageBox::warning (0, loc->translate("Warning"), 
	      loc->translate("That doesn't seem to be a real account!"));

	delete newSess;
	return (FALSE);
      }
	 
    // session with same name are not allowed
    sessionManager* iter;
    for ( iter=sess.first(); iter!=0; iter=sess.next() )
      {
	if (strcmp(newSess->profileName, iter->profileName)==0)       // they are equal
	  {
	    QMessageBox achtung (this);
	    achtung.message (loc->translate("Warning"), 
			     loc->translate("No 2 sessions with same name!"));

	    //	    newSess.session->setText ("");
	    //	    newSess.session->setFocus ();

	    delete newSess;
	    return (FALSE);
	  }
      }

    //
    // everything ok, start to append new session
    //
    int ID = getNewSessionID ();

    // sets this also to the current session
    sess.append (newSess);
    
    sess.current()->ID = ID;

    m->profileName->insertItem (sess.current()->profileName);

    // add it to toolbar
    if (sess.current()->addToToolbar == TRUE)
      {
	tools->insertButton (KApplication::getKApplication()->getIconLoader()->loadIcon (sess.current()->toolbarIcon), 
			     sess.current()->ID, TRUE, 
			     sess.current()->profileName, -1);
      }

    onlyChangeLines = FALSE;
    needSessionSaving (TRUE);

    restoreSession ();

    return (TRUE);
  }


// **************************  start a new session  **************************


void MyWidget::startNewSession ()
  {
    // first append it to the list ...    
    if (appendNewSession () == FALSE) return;

    // set the ComboBox on the last item ...
    m->profileName->setCurrentItem (sess.find(sess.last()));
    m->changeLines ();  
    
    startSession (sess.at());

    return;
  }


// *********************  now back to MyWidget  ******************************

// ********************  has one of the setups changed ?  ********************


void MyWidget::portHasChanged (const char* newPort)
{
  if (editNewSession == TRUE) return;

  sess.at(m->profileName->currentItem ())->port = atoi (newPort);
  needSessionSaving (TRUE);
}


void MyWidget::hostHasChanged (const char* newText)
  {
    if (editNewSession == TRUE) return;

    strcpy (sess.at(m->profileName->currentItem ())->host, newText);
    needSessionSaving (TRUE);
  }


void MyWidget::accountHasChanged (const char* newText)
  {
    if (editNewSession == TRUE) return;

    strcpy (sess.at(m->profileName->currentItem ())->account, newText);
    needSessionSaving (TRUE);
  }


void MyWidget::terminalHasChanged (int i)
{
  if (editNewSession == TRUE) return;

  needSessionSaving (TRUE);

  strcpy (sess.current()->terminal, m->terminal->text(i));
}


void MyWidget::textsizeHasChanged (int i)
{
  if (editNewSession == TRUE) return;

  needSessionSaving (TRUE);

  switch (i)
    {
    case 0: strcpy (sess.current()->font, "6x12");
      break;
    case 1: strcpy (sess.current()->font, "7x14");
      break;
    case 2: strcpy (sess.current()->font, "8x13");
      break;
    case 3: strcpy (sess.current()->font, "9x15");
      break;
    case 4: strcpy (sess.current()->font, "10x20");
    }
}


void MyWidget::connectorHasChanged (int i)
{
  if (editNewSession == TRUE) return;

  needSessionSaving (TRUE);

  switch (i)
    {
    case 0: 
      strcpy (sess.current()->connectionProgram, "telnet");
      break;
    case 1: 
      strcpy (sess.current()->connectionProgram, "rlogin");
      break;
    case 2: 
      strcpy (sess.current()->connectionProgram, "ssh");
    }
}


void MyWidget::fgcolorHasChanged (const QColor& newColor)
{
  if (editNewSession == TRUE) return;

  needSessionSaving (TRUE);

  strcpy (sess.current()->fgcolor, QColor2string (newColor));
}


void MyWidget::bgcolorHasChanged (const QColor& newColor)
{
  if (editNewSession == TRUE) return;

  needSessionSaving (TRUE);

  strcpy (sess.current()->bgcolor, QColor2string (newColor));
}


//CT 9jan98 following the introd of cmdline in the main widget
void MyWidget::cmdlineHasChanged (const char* newText)
{
  if (editNewSession == TRUE) return;

  needSessionSaving (TRUE);
 
  strcpy (sess.current()->commandLine, newText );
}


// *******************  show the mainwidget after the intro ******************


void MyWidget::showTheWidget ()
  {
    // the positions x and y are defined in the resource-file
      
    if (this->isVisible () == FALSE) this->show ();
 
    return;
  }  


// ***************************************************************************
//
// the two first slots only parse the session start to the third one.
// The argument can be a session ID, which starts on 100 + sessionNumber
// in the QList. So, I have to lower the ID by 100 in the first slot.
//
// The second slot starts from clicking on the connect button, so it starts
// the active session and it gets no sessionnumber from the pushbutton.
// So it has to find it for itself.
//


void MyWidget::startSession_ID (int ID)
{
  //
  // we have to find out the correct session from ID
  //
  sessionManager* iter;
  int oldCurrent = sess.at();

  for (iter=sess.first(); iter!=0; iter=sess.next())
    if (iter->ID == ID) break;

  startSession (sess.find (iter));

  sess.at(oldCurrent);
}


void MyWidget::startSession ()
{
  //
  // start current session
  //
  startSession (sess.at());
}


/*
 * only called by one of the two above slots
 *
 * This slot MUST NOT change the pointer to the current session, so I have to 
 * store it before beginning...
 *
 */

void MyWidget::startSession (int sessionNumber)
  {
    // change to the wanted sessionnumber
    int i;
    int oldCurrent = sess.at();
    sess.at (sessionNumber);

    // get config from main application
    KConfig* k = KApplication::getKApplication()->getConfig ();
    k->setGroup ("common");

    //
    // fork the process and store its process ID
    //
    int pid = fork ();

    if (k->readEntry ("callXhost", "FALSE") == QString ("TRUE"))
      {
	if (pid == 0)
	  {
	    // calling xhost
	    execlp ("xhost", "xhost", sess.current()->host, 0);
	  }

	// fork another time
	pid = fork ();
      }

    // the next forked process
    if (pid == 0)		  // the process with PID 0 will be replaced 
				  // by the terminal program	
      {
	char help1[50];
	char help2[50];

        // for the fontsize, recommended for the kvt terminal
        char textsize[40];

        // the colors, recommended for the kvt     
	char vgColText[10];
	char bgColText[10];

	// get memory for list of arguments
	char** cmdL = new char*[PARAMS];
	for (i=0; i<PARAMS;i++) cmdL[i] = new char[50];

	// doing intitializing, beware of kvt and xterm which want
	// different commandline options	

	// this order runs with all connection programs
	strcpy (help1, "-l");
	strcpy (help2, sess.current()->account);

	// now the terminal dependend commandline options
	strcpy (vgColText,"-");
	strcpy (bgColText,"-");
	strcpy (textsize,"-");
	strcat (vgColText, termFgColor.at(terminals.find(sess.current()->
							 terminal)));
	strcat (bgColText, termBgColor.at(terminals.find(sess.current()->
							 terminal)));
	strcat (textsize, termFontSize.at(terminals.find(sess.current()->
							 terminal)));

	//
	// write the commandLine parameters strings
	//
	strcpy (cmdL[0], sess.current()->terminal);
	strcpy (cmdL[1], textsize);
	strcpy (cmdL[2], sess.current()->font);
	strcpy (cmdL[3], "-T");
	strcpy (cmdL[4], sess.current()->profileName);
	strcpy (cmdL[5], vgColText);
	strcpy (cmdL[6], sess.current()->fgcolor);
	strcpy (cmdL[7], bgColText);
	strcpy (cmdL[8], sess.current()->bgcolor);

	//
	// split the optional commandLine in parts - 
	// quite ugly but should work without problems
	//
	i = sscanf (sess.current()->commandLine, "%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s", cmdL[9], cmdL[10], cmdL[11], cmdL[12], cmdL[13], cmdL[14], cmdL[15], cmdL[16], cmdL[17], cmdL[18], cmdL[19], cmdL[20], cmdL[21], cmdL[22], cmdL[23], cmdL[24], cmdL[25], cmdL[26], cmdL[27], cmdL[28]); 

	// 
	// this result is arrived if the commandLine is empty
	//
	if (i == -1) i = 0;

	// 
	// ******  now start the session  ******
	//	
	if ((strcmp (sess.current()->account, "-") == 0) 
	    && (strcmp (sess.current()->host, "-") == 0))
	  {
	    cmdL [9+i] = 0;

	    execvp (sess.current()->terminal, cmdL);
	  }
	else
	  {
	    strcpy (cmdL[9+i], "-e");
	    strcpy (cmdL[10+i], sess.current()->connectionProgram);

	    if (strlen (help2) != 0) 
	      {
		// only copy if account name is given
		strcpy (cmdL[11 + i++], help1);
		strcpy (cmdL[11 + i++], help2);
	      }

	    // first check if line has an entry, then choose the port
	    if (sess.current()->port != 0)
	      {
		// keep in mind the different commandline options
		if (strcmp(sess.current()->connectionProgram, "ssh") == 0)
		  {
		    strcpy (cmdL[11 + i++], "-p");
		    sprintf (cmdL[11 + i++], "%d", sess.current()->port);
		  }
	      }

	    // the host name
	    strcpy (cmdL[11 + i++], sess.current()->host);

	    // telnet wants the port number on the end
	    if (strcmp(sess.current()->connectionProgram, "telnet") == 0)
	      if (sess.current()->port != 0)
		sprintf (cmdL[11 + i++], "%d", sess.current()->port);

	    // the last entry: "NULL"
	    cmdL[11 + i++] = 0;

	    // here this part(fork) of the programs finishes
	    execvp (sess.current()->terminal, cmdL);
	  }
	
	// the program never reaches this point, I know, ...
	for (i=0; i<PARAMS; i++) delete [] cmdL[i];
	delete [] cmdL;
      }

    // reset the current session pointer
    sess.at (oldCurrent);
    
    if (k->readEntry ("exitOnConnect", "FALSE") == QString ("TRUE")) exit (0);
  }


// slot called to update the toolbar and menu entries for the "save" topic

void MyWidget::needSessionSaving (bool save)
{
  // show the save button in the toolbar or hide it
  if (onlyChangeLines == FALSE)
    {
      // only change when the signal is real
      tools->setItemEnabled (3, save);
      filemenue->setItemEnabled (filemenue->idAt(2), save);
    }

  needSaving = save;
}


//
// CT redefined to spawn up the new way of new seesion.
//

void MyWidget::newSessionSelect ()
{

  //store it because some slots will be called which should do nothing here
  onlyChangeLines = TRUE;
  neededSavingbefore = needSaving;

  // changes in lineedits and comboboxes should not be stored
  editNewSession = TRUE;
  
  //CT 9jan98 save initial window status
  if(m->height()==125) prev.status = 1;
  else 
    if (m->height()>125) prev.status = 2;
    else prev.status = 0;
  prev.session = sess.current();
  
  m->showFull();
  
  m->profileName->hide ();
  m->newSessionLined->setText("");
  m->newSessionLined->show ();
  m->restoreoldbutton->show ();
  m->savenewbutton->show ();
  m->Connect->hide ();

  // disallow some things within toolbar and popup menus
  deactivateSLOTS ();

  disconnect (m->toolbariconbutton, SIGNAL (iconChanged(const char*)), m,
	      SLOT (changeIcon(const char*)));

  disconnect (m->addtotoolbar, SIGNAL (clicked()), m, SLOT (slotToolbar ()));
  connect (m->addtotoolbar, SIGNAL (clicked()), SLOT (slotToolbar ()));

  // disconnect the resize buttons
  disconnect (m->buttonUp, SIGNAL (clicked ()), m, SLOT (reduceSize ()));
  disconnect (m->buttonDown, SIGNAL (clicked ()), m, SLOT (increaseSize ()));
  m->buttonUp->setPixmap (KApplication::getKApplication()->getIconLoader()->loadIcon("smallup_grey.xpm"));

  m->options->setText( "" );
  m->terminal->setCurrentItem(0);
  m->textsize->setCurrentItem (3);
  m->connector->setCurrentItem (0);      // telnet means ... (next line)
  m->port->setEnabled(TRUE);
  m->port->setValue (23);
  m->addtotoolbar->setText( loc->translate("Add to toolbar"));
  
  m->host->setText( "" );
  m->account->setText( "" );
  m->fgcolorbutton->setColor (QColor(0, 0, 0));
  m->bgcolorbutton->setColor (QColor (255, 255, 255));
  m->toolbariconbutton->setIcon ("session1.xpm");
     
  m->newSessionLined->setFocus();

  NewSessAccel = new QAccel (this);
  NewSessAccel->connectItem( NewSessAccel->insertItem(Key_Escape), 
			     this,     
			     SLOT(restoreSession()) );
  NewSessAccel->connectItem( NewSessAccel->insertItem(Key_Return), 
			     this,     
			     SLOT(appendNewSession()) );

  // restore and make clear, that the toolbar looks nice
}


void MyWidget::slotToolbar ()
{
  if (strcmp (m->addtotoolbar->text(), loc->translate("Add to toolbar")) == 0)
    m->addtotoolbar->setText( loc->translate("Remove"));
  else  m->addtotoolbar->setText( loc->translate("Add to toolbar"));
}


void MyWidget::deleteSessionSelect ()
  {
    if (sess.count () == 1)
      {
	QMessageBox achtung (this);
	achtung.message (loc->translate("Error"), 
			 loc->translate("You need at least 1 session!"));

	return;
      }

    QString whatToSay = loc->translate("You attempt to delete session \"");
    whatToSay += sess.current()->profileName;
    whatToSay += loc->translate("\"!\n\nContinue deleting?");

    int result = QMessageBox::warning (0, loc->translate("Delete session"), 
		    (const char*)whatToSay, loc->translate("Yes"), 
				       loc->translate("No"), 0, 1);

    if (result == 0) deleteSession ();
 
    return;
  }


// ***************************************************************************


//
// check for the need to save the configuration before closing the main widget
//

void MyWidget::closeEvent (QCloseEvent* e)
  {
    KConfig* k = KApplication::getKApplication()->getConfig();
    k->setGroup ("common");

    if (k->readEntry ("autosaveOptions", "FALSE") == QString ("TRUE") ) 
      {
	k->sync ();      
      }
    
    if (needSaving == TRUE) 
      {
	if (k->readEntry("autosaveSessions", "FALSE") == QString("TRUE"))
	  {
	    saveConfig ();
	    e->accept ();
	    return;
	    // here we leave the application
	  }

	//
	// no autosave, continue with dialog
	//

	int result = QMessageBox::warning (0, loc->translate("Warning"), 
	  loc->translate("Your sessions have changed but are not saved yet.\nSave now?"),loc->translate("Yes"), loc->translate("No"), 
					   loc->translate("Cancel"), 2, 2); 

	if (result == 0)                        // Yes
	  {
	    saveConfig ();
	    e->accept ();
	  }
	else if (result == 1) e->accept ();     // No 
	else e->ignore ();                      // Cancel
      }
    else e->accept ();
  }


//
// check for the need to save the configuration before exit...
//

void MyWidget::endApplication ()
  {
    //    saveProperties (KApplication::getKApplication()->getConfig());

    KConfig* k = KApplication::getKApplication()->getConfig();
    k->setGroup ("common");

    if (k->readEntry ("autosaveOptions", "FALSE") == QString ("TRUE")) 
      {
	k->sync ();      
      }

    if (needSaving == TRUE) 
      {
	if (k->readEntry("autosaveSessions", "FALSE") == QString("TRUE"))
	  {
	    saveConfig ();
	    KApplication::getKApplication()->quit();
	    return;
	    // here we leave the application
	  }
	
	//
	// no autosave, continue with dialog
	//

	int result = QMessageBox::warning (0, loc->translate("Warning"), 
	   loc->translate("Your sessions have changed but are not saved yet.\nSave now?"),
	   loc->translate("Yes"), loc->translate("No"), 
	   loc->translate("Cancel"), 2, 2); 

	if (result == 0)                                              // Yes
	  {
	    saveConfig ();
	    KApplication::getKApplication()->quit();
	  }
	else if (result == 1) KApplication::getKApplication()->quit();// No
	else ;                                                        // Cancel
      }

    else 
      {
	KApplication::getKApplication()->quit();
      }
  }


void MyWidget::hideMenuBar ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");
  k->writeEntry ("showMenuBar", "FALSE");

  menue->hide ();
  optionMenue->removeItemAt (0);
  optionMenue->insertItem (loc->translate("Show &menubar"), -1, 0);
  optionMenue->connectItem (optionMenue->idAt (0), this, SLOT(showMenuBar ()));

  updateRects ();
}


void MyWidget::showMenuBar ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");
  k->writeEntry ("showMenuBar", "TRUE");

  menue->show ();
  optionMenue->removeItemAt (0);
  optionMenue->insertItem (loc->translate("Hide &menubar"), -1, 0);
  optionMenue->connectItem (optionMenue->idAt (0), this, SLOT(hideMenuBar ()));

  updateRects ();
}


void MyWidget::hideToolBar ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");
  k->writeEntry ("showToolBar", "FALSE");

  tools->hide ();
  optionMenue->removeItemAt (1);
  optionMenue->insertItem (loc->translate("Show &toolbar"), -1, 1);
  optionMenue->connectItem (optionMenue->idAt (1), this, SLOT(showToolBar ()));

  updateRects ();
}


void MyWidget::showToolBar ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");
  k->writeEntry ("showToolBar", "TRUE");

  tools->show ();
  optionMenue->removeItemAt (1);
  optionMenue->insertItem (loc->translate("Hide &toolbar"), -1, 1);
  optionMenue->connectItem (optionMenue->idAt (1), this, SLOT(hideToolBar ()));

  updateRects ();
}


void MyWidget::showOptionsPopup (QPoint p)
{
  QRect hilf = geometry ();

  optionMenue->popup ( p + hilf.topLeft () + m->pos());
}


void MyWidget::showFilePopup (QPoint p)
{
  QRect hilf = geometry ();

  filemenue->popup ( p + hilf.topLeft () + m->pos());
}


// *********************  adding icons to toolbar  ****************************


void MyWidget::addToToolbar ()
{
  needSessionSaving (TRUE);

  sess.current()->addToToolbar = TRUE;

  //
  // always put the icons in the same order in the toolbar, so remove
  // all and insert all afterwards
  //
  sessionManager* iter;
  int oldCurrent = sess.at();

  for (iter=sess.first (); iter!=0; iter=sess.next())
    {
      tools->removeItem (iter->ID);
    }

  for (iter=sess.first (); iter!=0; iter=sess.next())
    {
      if (iter->addToToolbar == TRUE)
	tools->insertButton (KApplication::getKApplication()->getIconLoader()->
			     loadIcon (iter->toolbarIcon), 
			     iter->ID, TRUE, 
			     iter->profileName, -1);
    }
  sess.at (oldCurrent);
}


void MyWidget::removeFromToolbar ()
{
  needSessionSaving (TRUE);

  sess.current()->addToToolbar = FALSE;

  tools->removeItem (sess.current()->ID);     // remove a certain ID
}


void MyWidget::setup ()
{
  configTab* setup = new configTab (this, 0, TRUE);
  setup->show ();
}


void MyWidget::saveOptions ()
{
  KConfig* k = KApplication::getKApplication ()->getConfig();

  k->sync ();
}


void MyWidget::showMinimum ()
{
  m->showMinimum ();
}


void MyWidget::showFull ()
{
  m->showFull ();
}


void MyWidget::showStandard ()
{
  m->showStandard ();
}


void MyWidget::showHelper ()
{
  KApplication::getKApplication()->invokeHTMLHelp ("","");
}


MyWidget::MyWidget (QWidget*, const char* name)
  : KTopLevelWidget (name)
  {
    KConfig* k = KApplication::getKApplication()->getConfig();
    k->setGroup ("terminal");
    k->readListEntry ("terminals", terminals);
    if (terminals.isEmpty() == TRUE) 
      {
        QMessageBox::warning (0, loc->translate("Warning"), 
           loc->translate("Something went wrong reading the global resource file! Could not find any terminals. Stop!")); 	
	exit (0);       
      }

    // different terminals need different commandline options
    k->readListEntry ("termfontsize", termFontSize);
    k->readListEntry ("termfgcolor", termFgColor);
    k->readListEntry ("termbgcolor", termBgColor);

    // of course, at beginning no new session is edited
    editNewSession = FALSE;

    // define the pointers of the other widgets to NULL
    //    n = 0;

    // as it is: read configuration
    readConfigurationFile ();

    // set the first item to be current

    // ****************************  Menue  **********************************
    
    menue = new KMenuBar (this);
    m = new mainWidget (this);
    tools = new KToolBar (this);

    // ************************  the toolbar  *******************************

    // load the icons
    KIconLoader* kiL = KApplication::getKApplication()->getIconLoader();

    tools->insertButton (kiL->loadIcon ("exit.xpm"), 0, SIGNAL (clicked()), 
			 this, SLOT (endApplication()), TRUE, 
			 loc->translate("Exit the remote login manager"));
    tools->insertSeparator ();
    tools->insertButton (kiL->loadIcon ("filenew.xpm"), 1, SIGNAL (clicked()), 
			 this, SLOT (newSessionSelect()), TRUE, 
			 loc->translate("Define new session"));
    tools->insertButton (kiL->loadIcon ("editcut.xpm"), 2, SIGNAL (clicked()), 
			 this, SLOT (deleteSessionSelect()), TRUE, 
			 loc->translate("Delete session"));
    tools->insertButton (kiL->loadIcon ("filefloppy.xpm"), 3, 
			 SIGNAL (clicked()), 
			 this, SLOT (saveConfiguration()), TRUE, 
			 loc->translate("Save sessions"));
    tools->insertSeparator ();
    tools->insertButton (kiL->loadIcon ("help.xpm"), 0, SIGNAL (clicked()), 
			 this, SLOT (showHelper()), TRUE, 
			 loc->translate("Get Help"));

    //
    // now insert the session buttons
    //
    tools->insertSeparator ();

    sessionManager* iter;

    for (iter=sess.first(); iter!=0; iter=sess.next())
      {
	if (iter->addToToolbar == TRUE) 
	  tools->insertButton (kiL->loadIcon (iter->toolbarIcon), 
			       iter->ID, TRUE, 
			       iter->profileName);
      }

    connect ( tools, SIGNAL (clicked (int)), SLOT (startSession_ID(int)) );

    // ************************  new Popupmenus  ****************************  

    filemenue = new QPopupMenu; CHECK_PTR (filemenue);
    optionMenue = new QPopupMenu; CHECK_PTR (optionMenue);

    // ************************  now the option menue  ************************

    optionMenue->insertItem (loc->translate("Hide &menubar"));
    optionMenue->insertItem (loc->translate("Hide &toolbar"));
    optionMenue->insertSeparator ();
    optionMenue->insertItem (loc->translate("&Options"), this, SLOT (setup ()) );
    optionMenue->insertItem (loc->translate("Sa&ve options"), this, SLOT (saveOptions ()) );

    // ************************ the file menue ********************************

    filemenue->insertItem (loc->translate("&New Session"), this, SLOT (newSessionSelect ()), CTRL+Key_N );
    filemenue->insertItem (loc->translate("&Delete Session"), this, 
			   SLOT (deleteSessionSelect ()) );
    filemenue->insertItem (loc->translate("&Save Sessions"), this, 
			   SLOT (saveConfiguration ()), CTRL+Key_S );
    filemenue->insertSeparator ();
    filemenue->insertItem (loc->translate("E&xit"), this, SLOT (endApplication ()), CTRL+Key_Q );
 
    // ****************  the help menue is very short  ************************

    char* help = new char [400];
    strcpy (help, "KTelnet  ");
    strcat (help, VERSION);
    strcat (help, loc->translate("     formerly Telly"));
    strcat (help, 
	    loc->translate("\n\n(c) 1997/98 by\nSpaghetti Code Computing"));
    strcat (help, 
	  loc->translate("\n\nWritten by Martin Hfner and Cristian Tibirna"));
    strcat (help, 
	   loc->translate("\n\nEmail: mh@ap-dec717c.physik.uni-karlsruhe.de"));
    strcat (help, loc->translate("\nHomepage: http://ap-dec717c.physik.uni-karlsruhe.de/~mh/ktelnet"));
    strcat (help, loc->translate("\n\nThanks to all translators!"));

    helpmenue = KApplication::getKApplication()->getHelpMenu (TRUE, 
					   loc->translate(help));
    delete help;

    // ***************** now insert the Popups into the menuebar **************

    menue->insertItem (loc->translate("&File"), filemenue);
    menue->insertItem (loc->translate("&Options"), optionMenue);
    menue->insertSeparator ();
    menue->insertItem (loc->translate("&Help"), helpmenue);

    // set up the menu entries
    sess.at(0);
    m->changeLines ();

    // now insert all widgets to KTopLevelWidget
    setMenu (menue);
    addToolBar (tools);
    setView (m);

    // now after inserting the toolbar calling this as at this point of the
    // file nothing will be needed to save
    needSessionSaving (FALSE);
    
    // watch on the common configuration file
    k->setGroup ("common");
    
    if (k->readEntry ("showMenuBar", "TRUE") == QString ("TRUE")) 
      showMenuBar ();
    else hideMenuBar ();
    if (k->readEntry ("showToolBar", "TRUE") == QString ("TRUE")) 
      showToolBar ();	
    else hideToolBar ();

    if (k->readEntry ("windowSize", "standard") == QString ("minimum")) 
      m->showMinimum ();
    else if (k->readEntry ("windowSize", "standard") == QString ("standard")) 
      m->showStandard ();
    else m->showFull ();

    //CT 9jan98 set previousStatus for new
    prev.status = 1;
    prev.session = sess.current();
  }


MyWidget::~MyWidget ()
{
  delete tools;

  delete filemenue;
  delete optionMenue;
  delete helpmenue;
}








