#ifndef __MYWIDGET_H

#define __MYWIDGET_H


#define PARAMS 40
#define SESSIONSFILE "ktelnet.sessions"


#include <qwidget.h>

#include <kmenubar.h>
#include <kcolordlg.h>
#include <ktopwidget.h>
#include <ktoolbar.h>
#include <dataclass.h>



class QPushButton;
class QComboBox;
class QLineEdit;
class QPixmap;
class QPopupMenu;
class QTimer;
class QLCDNumber;
class QSlider;
class MyWidget;



//
// 2 helper-functions
//

QColor string2QColor (const char* color);
char* QColor2string (QColor col);


// this the main widget of the application

class MyWidget : public KTopLevelWidget
  {
    friend class mainWidget;
    Q_OBJECT

    public:
      MyWidget ( QWidget* parent=0, const char* name=0);
      ~MyWidget ();

    public slots:
      void deactivateSLOTS ();
      void activateSLOTS ();

      void needSessionSaving (bool);

      void showTheWidget ();     // after some delay where the intro is visible

      void hostHasChanged (const char* newText);     // lots of nasty slots...
      void accountHasChanged (const char* newText);
      void terminalHasChanged (int);
      void textsizeHasChanged (int);
      void connectorHasChanged (int);
      void bgcolorHasChanged (const QColor&);
      void fgcolorHasChanged (const QColor&);
      void cmdlineHasChanged (const char* newText); //CT 9jan98
      void portHasChanged (const char* newPort);

      void setup ();
      void saveOptions ();
      
      bool appendNewSession ();
      void restoreSession (); 
      void startNewSession ();
      void newSessionSelect ();

      void deleteSession ();
      void deleteSessionSelect ();

      void readConfigurationFile ();         
      void saveConfiguration (); 

      void showHelper ();

      //
      // one starts the active session when clicked on "Connect", the other
      // starts the session with the index int in the list
      //
      void startSession ();
      void startSession_ID (int);
      void startSession (int);

      void removeFromToolbar ();
      void addToToolbar ();

      // only parse it to the mainwidget "m"
      void showMinimum ();
      void showStandard ();
      void showFull ();

      void hideMenuBar ();
      void showMenuBar ();

      void showToolBar ();
      void hideToolBar ();

      void showOptionsPopup (QPoint);
      void showFilePopup (QPoint);

    protected:
      void closeEvent (QCloseEvent*);

    private slots:
      void endApplication ();
      void slotToolbar ();

    private:
      void saveConfig ();

      bool editNewSession;
     
      KMenuBar* menue;
      KToolBar* tools;

      mainWidget* m;

      QPopupMenu* filemenue;
      QPopupMenu* optionMenue;
      QPopupMenu* helpmenue;

      QAccel* NewSessAccel;

      struct previousStatus
      {
	int status;			   //CT 9jan98 needed to save the
	sessionManager* session;		   // main widget's status
      } prev;				   // 0=min;1=standard;2=max
  };


// further docu see in mywidget.cc
extern bool needSaving;      
extern bool onlyChangeLines;

#endif





